//-----------------------------------------------------------
//    eg.cpp
//-----------------------------------------------------------
#include "eg.h"
#include <math.h>
//-----------------------------------------------------------
//    
//-----------------------------------------------------------
const unsigned int default_attack_count = 5000;
const unsigned int default_decay_count = 5000;
const unsigned int default_release_count = 5000;
const float default_sustain_level = 0.5;
//-----------------------------------------------------------
//    
//-----------------------------------------------------------
SynthEG::SynthEG()
{
	phase = 0;
	state1 = NON;
	SetAttackCount(default_attack_count);
	SetDecayCount(default_decay_count);
	SetReleaseCount(default_release_count);
	SetSustainLevel(default_sustain_level);
	release_level = sustain_level;
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
SynthEG::~SynthEG()
{
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
EGState SynthEG::GetEGState() const
{
	return state1;
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void SynthEG::SetEGState(const EGState egs1)
{
	if(egs1 == RELEASE){
		release_level = pre_val;
	}
	state1 = egs1;
	phase = 0;
}




//-----------------------------------------------------------
//    
//-----------------------------------------------------------
float SynthEG::GetValue()
{
	float val, n1;
	switch(GetEGState()){
		case ATTACK:
			val = (attack_rate * (float)phase);
			phase++;
			if(phase==attack_count) SetEGState(DECAY);
			pre_val = val;
			return (val);
		case DECAY:
			n1 = delta_phase_decay * (float)phase;
			val = (1.0 - sustain_level) * inv_exp_table[(int)n1];
			val += sustain_level;
			phase++;
			if(phase==decay_count) SetEGState(SUSTAIN);
			pre_val = val;
			return (val);
		case SUSTAIN:
			pre_val = sustain_level;
			return (sustain_level);
		case RELEASE:
			n1 = delta_phase_release * (float)phase;
			val = release_level * inv_exp_table[(int)n1];
			phase++;
			if(phase==release_count) SetEGState(NON);
			pre_val = val;
			return (val);
		case NON:
			pre_val = 0.0;
			return (0.0);
	}
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void SynthEG::ResetPhase()
{
	phase = 0;
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void SynthEG::SetAttackCount(const int acount)
{
	if(acount <= 0) return;
	attack_count = acount;
	attack_rate = 1.0 / (float)attack_count;
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void SynthEG::SetDecayCount(const int dcount)
{
	if(dcount <= 0) return;
	decay_count = dcount;
	delta_phase_decay = (float)inv_exp_table_size / (float)decay_count;
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void SynthEG::SetReleaseCount(const int rcount)
{
	if(rcount <= 0) return;
	release_count = rcount;
	delta_phase_release = (float)inv_exp_table_size / (float)release_count;
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void SynthEG::SetSustainLevel(const float slevel)
{
	if((slevel < 0.0)||(slevel > 1.0)) return;
	sustain_level = slevel;
}

