//-----------------------------------------------------------
//    signal_process_base.cpp
//-----------------------------------------------------------
#include "signal_process_base.h"
//-----------------------------------------------------------
//    const
//-----------------------------------------------------------
const int default_duration_nap = 100;
//-----------------------------------------------------------
//    static member
//-----------------------------------------------------------
std::vector<unsigned int> SignalProcessBase::id_record;
std::vector<SignalProcessBase*> SignalProcessBase::spb_record;
const unsigned int SignalProcessBase::sample_rate = 44100;
const unsigned int SignalProcessBase::size_buf = 4410;
//-----------------------------------------------------------
//    constructor
//-----------------------------------------------------------
SignalProcessBase::SignalProcessBase(int id1)
{
	duration_nap =
		boost::posix_time::microseconds(default_duration_nap);
	buffer = new float[size_buf];
	for(int i1 = 0; i1 < size_buf; i1++){
		buffer[i1] = 0.0;
	}
	working = false;
	running_routine = false;
	
	if(id1<0){
		if(id_record.size()==0){
			id = 0;
			id_record.push_back(id);
		}else{
			int itmp = 0;
			for(int i1 = 0, sz = id_record.size(); i1 < sz; i1++){
				if(itmp < id_record[i1]) itmp = id_record[i1];
			}
			itmp++;
			id = itmp;
			id_record.push_back(itmp);
		}
	}else{
		int itmp = 0;
		bool id_exist = false;
		for(int i1 = 0, sz = id_record.size(); i1 < sz; i1++){
			if(itmp < id_record[i1]) itmp = id_record[i1];
			id_exist |= (id_record[i1] == id1);
		}
		itmp++;
		
		if(id_exist){
			id = itmp;
			id_record.push_back(itmp);
		}else{
			id = id1;
			id_record.push_back(id1);
		}
	}
	spb_record.push_back(this);
	StartRoutine();
	noteon_count = -1;
	noteoff_count = -1;
	uplimit = 1;
	downlimit = 1;
}

//-----------------------------------------------------------
//    destructor
//-----------------------------------------------------------
SignalProcessBase::~SignalProcessBase()
{
	StopRoutine();
	delete [] buffer;
	int itmp = GetId();
	for(int i1 = 0, sz = id_record.size(); i1 < sz; i1++){
		if(id_record[i1]==itmp){
			id_record.erase(id_record.begin()+i1);
			spb_record.erase(spb_record.begin()+i1);
			break;
		}
	}
}

//-----------------------------------------------------------
//    Routine
//-----------------------------------------------------------
void SignalProcessBase::Routine()
{
	while (1)
	{
		if(!running_routine) break;
		if(!HasTask()){
			boost::this_thread::sleep<boost::posix_time::time_duration>(duration_nap);
		}else{
			boost::mutex::scoped_lock lk(guard);
			Work();
		}
	}
}

void SignalProcessBase::StartRoutine()
{
	boost::mutex::scoped_lock lk(guard);
	if (running_routine) return;
	running_routine = true;
	part_thread = new boost::thread(&SignalProcessBase::Routine, this);
}

void SignalProcessBase::StopRoutine()
{
	boost::mutex::scoped_lock lk(guard);
	if(!running_routine) return;
	running_routine = false;
	bool done = false;
	lk.unlock();
	while (!done) {
		//part_thread->interrupt();
		done = part_thread->timed_join(boost::posix_time::millisec(10));
	}
	delete part_thread;
	part_thread = 0;
}

bool SignalProcessBase::IsRunning() const
{
	return running_routine;
}

//-----------------------------------------------------------
//    Work
//-----------------------------------------------------------
void SignalProcessBase::Work()
{
	
}

//-----------------------------------------------------------
//    Type
//-----------------------------------------------------------
SignalProcessType  SignalProcessBase::GetType() const
{
	return sptype;
}

void SignalProcessBase::SetSignalProcessType(const SignalProcessType tp1)
{
	sptype = tp1;
}

//-----------------------------------------------------------
//    Buffer
//-----------------------------------------------------------
const float* SignalProcessBase::GetBuffer() const
{
	return buffer;
}

void SignalProcessBase::CopyBuffer(float* cbuf) const
{
	for(int i1 = 0; i1 < size_buf; i1++){
		cbuf[i1] = buffer[i1];
	}
}

unsigned int SignalProcessBase::GetSizeBuffer() const
{
	return size_buf;
}
void SignalProcessBase::InitializeBuffer()
{
	for(int i1 = 0; i1 < size_buf; i1++){
		buffer[i1] = 0.0;
	}
}

void SignalProcessBase::WriteBuffer(const float* d1)
{
	for(int i1 = 0; i1 < size_buf; i1++){
		buffer[i1] = d1[i1];
	}
}

//-----------------------------------------------------------
//    Task
//-----------------------------------------------------------
bool SignalProcessBase::HasTask() const
{
	if(tasks.size()==0) return false;
	else return (tasks.count()==tasks.size());
}

void SignalProcessBase::AccomplishTask()
{
	tasks.reset();
}

void SignalProcessBase::TakeTask(unsigned int id1)
{
	boost::mutex::scoped_lock lk(guard);
	for(int i1 = 0; i1 < upstream.size(); i1++){
		if(id1==upstream[i1]->GetId()){
			tasks.set(i1);
		}
	}
}

void SignalProcessBase::GiveTask()
{
	for(int i1 = 0; i1 < downstream.size(); i1++){
		downstream[i1]->TakeTask(GetId());
	}
}

void SignalProcessBase::SetTask()
{
	tasks.set();
}

//-----------------------------------------------------------
//    UpStream
//-----------------------------------------------------------
void SignalProcessBase::SetUpStream(SignalProcessBase* spb1)
{
	upstream.clear();
	upstream.push_back(spb1);
	tasks.resize(1);
	tasks.reset();
}

void SignalProcessBase::SetUpStream(const std::vector<SignalProcessBase*>& spbs1)
{
	if(uplimit==-1){
		upstream = spbs1;
		tasks.resize(upstream.size());
		tasks.reset();
	}else{
		int i1;
		upstream.clear();
		for(i1 = 0; (i1 < uplimit)&&(i1 < spbs1.size()); i1++){
			upstream.push_back(spbs1[i1]);
		}
		tasks.resize(i1);
		tasks.reset();
	}
}

const SignalProcessBase* SignalProcessBase::GetUpStream(unsigned int upn) const
{
	if(upn>=upstream.size()) return 0;
	else return upstream[upn];
}

const std::vector<SignalProcessBase*>& SignalProcessBase::GetUpStream() const
{
	return upstream;
}

void SignalProcessBase::SetUpLimit(const int ulim)
{
	uplimit = ulim;
}

//-----------------------------------------------------------
//    DownStream
//-----------------------------------------------------------
void SignalProcessBase::SetDownStream(SignalProcessBase* spb1)
{
	downstream.clear();
	downstream.push_back(spb1);
}

void SignalProcessBase::SetDownStream(const std::vector<SignalProcessBase*>& spbs1)
{
	if(downlimit==-1) downstream = spbs1;
	else{
		downstream.clear();
		for(int i1 = 0; (i1 < downlimit)&&(i1 < spbs1.size()); i1++){
			downstream.push_back(spbs1[i1]);
		}
	}
}

const SignalProcessBase* SignalProcessBase::GetDownStream(unsigned int dnm) const
{
	if(dnm>=downstream.size()) return 0;
	else return downstream[dnm];
}

const std::vector<SignalProcessBase*>& SignalProcessBase::GetDownStream() const
{
	return downstream;
}

void SignalProcessBase::SetDownLimit(const int dlim)
{
	downlimit = dlim;
}

//-----------------------------------------------------------
//    other
//-----------------------------------------------------------
unsigned int SignalProcessBase::GetId() const
{
	return id;
}

unsigned int SignalProcessBase::GetSampleRate() const
{
	return sample_rate;
}

void SignalProcessBase::SetNoteOnCount(int count)
{
	noteon_count = count;
}

void SignalProcessBase::SetNoteOffCount(int count)
{
	noteoff_count = count;
}

int SignalProcessBase::GetNoteOnCount() const
{
	return noteon_count;
}

int SignalProcessBase::GetNoteOffCount() const
{
	return noteoff_count;
}

const std::vector<SignalProcessBase*>& SignalProcessBase::GetSignalProcesses() const
{
	return spb_record;
}
