//-----------------------------------------------------------
//    SignalProcessBase
//-----------------------------------------------------------
#ifndef __SIGNAL_PROCESS_BASE_H__
#define __SIGNAL_PROCESS_BASE_H__
#include <boost/thread.hpp>
#include <boost/signals2.hpp>
#include <boost/dynamic_bitset.hpp>
#include <iostream>
//-----------------------------------------------------------
//    enum SignalProcessType
//-----------------------------------------------------------
enum SignalProcessType{VCO, VCF, VCA, MIXER, TO_PA};
extern const int default_size_buf;
extern const int default_duration_nap;
extern const int default_sample_rate;
//-----------------------------------------------------------
//    class SynthBase
//-----------------------------------------------------------
class SignalProcessBase
{
public:
	SignalProcessBase(int id1 = -1);
	virtual ~SignalProcessBase();
	void StopRoutine();
	void StartRoutine();
	virtual bool HasTask() const;
	virtual void TakeTask(unsigned int id1);
	virtual void GiveTask();
	virtual void AccomplishTask();
	const float* GetBuffer() const;
	void CopyBuffer(float* cbuf) const;
	SignalProcessType GetType() const;
	unsigned int GetSizeBuffer() const;
	void InitializeBuffer();
	void WriteBuffer(const float* d1);
	virtual void SetUpStream(SignalProcessBase* spb1);
	virtual void SetUpStream(const std::vector<SignalProcessBase*>& spbs1);
	virtual void SetDownStream(SignalProcessBase* spb1);
	virtual void SetDownStream(const std::vector<SignalProcessBase*>& spbs1);
	const SignalProcessBase* GetUpStream(unsigned int upn) const;
	const std::vector<SignalProcessBase*>& GetUpStream() const;
	const SignalProcessBase* GetDownStream(unsigned int dnm) const;
	const std::vector<SignalProcessBase*>& GetDownStream() const;
	const std::vector<SignalProcessBase*>& GetSignalProcesses() const;
	unsigned int GetId() const;
	unsigned int GetSampleRate() const;
	bool IsRunning() const;
	void SetUpLimit(const int ulim);
	void SetDownLimit(const int dlim);
	void SetSignalProcessType(const SignalProcessType tp1);
	void SetTask();
	void SetNoteOnCount(int count);
	void SetNoteOffCount(int count);
	int GetNoteOnCount() const;
	int GetNoteOffCount() const;
private:
	SignalProcessType sptype;
	unsigned int id;
	static const unsigned int sample_rate;
	int uplimit;
	int downlimit;
	int noteon_count;
	int noteoff_count;
	virtual void Routine();
	virtual void Work();
	static const unsigned int size_buf;
	float* buffer;
	std::vector<SignalProcessBase*> upstream;
	std::vector<SignalProcessBase*> downstream;
	bool working;
	bool running_routine;
	boost::dynamic_bitset<> tasks;
	boost::mutex guard;
	boost::thread *part_thread;
	boost::posix_time::time_duration duration_nap;
	static std::vector<unsigned int> id_record;
	static std::vector<SignalProcessBase*> spb_record;
};
#endif

