//-----------------------------------------------------------
//    to_portaudio.cpp
//-----------------------------------------------------------
#include "to_portaudio.h"
#include <math.h>
#include <portaudio.h>

const unsigned int default_frames_per_buffer = 64;
//-----------------------------------------------------------
//    
//-----------------------------------------------------------
ToPortAudio::ToPortAudio(int id1)
	: SignalProcessBase(id1)
{
	const unsigned int bsz = GetSizeBuffer();
	buffer_a = new float[bsz];
	buffer_b = new float[bsz];
	for(int i1 = 0; i1 < bsz; i1++){
		buffer_a[i1] = 0.0;
		buffer_b[i1] = 0.0;
	}
	initialized_portaudio = false;
	playing = false;
	state_changed = false;
	frames_per_buffer = default_frames_per_buffer;
	phase = 0;
	state1 = STATE_A;
	SetSignalProcessType(TO_PA);
	SetDownLimit(-1);
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
ToPortAudio::~ToPortAudio()
{
	
	delete [] buffer_a;
	delete [] buffer_b;
	stop_pa();
	close_pa();
}
//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void ToPortAudio::Work()
{
	float btmp[GetSizeBuffer()];
	
	if(!initialized_portaudio){
		if(state1==STATE_A){
			GetUpStream(0)->CopyBuffer(buffer_a);
		}else{
			GetUpStream(0)->CopyBuffer(buffer_b);
		}
		int err = init_pa(), err2 = 0;
		if(err != -1) err2 = start_pa();
		GiveTask();
	}else{
		if(state1==STATE_A){
			GetUpStream(0)->CopyBuffer(buffer_b);
		}else{
			GetUpStream(0)->CopyBuffer(buffer_a);
		}
	}
	AccomplishTask();
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
int ToPortAudio::PortAudioCallback(
	const void *inputBuffer, void *outputBuffer,
	unsigned long framesPerBuffer,
	const PaStreamCallbackTimeInfo* timeInfo,
	PaStreamCallbackFlags statusFlags,
	void *userData)
{
	ToPortAudio *data = (ToPortAudio*)userData;
	float *buf1;
	float *out = (float*)outputBuffer;
	int finished = 0;
	(void) timeInfo;
	(void) statusFlags;
	(void) inputBuffer;
	
	if(data->state1==STATE_A) buf1 = data->buffer_a;
	else buf1 = data->buffer_b;
	
	int n1 = 0;
	int bsz = data->GetSizeBuffer();
	for(int i1 = 0; i1 < data->frames_per_buffer; i1++){
		float c1 = (float)buf1[data->phase];
		out[n1] = c1;
		out[n1+1] = c1;
		n1 += 2;
		data->phase++;
		if(data->phase>=bsz){
			if(data->state1==STATE_A){
				data->state1 = STATE_B;
				buf1 = data->buffer_b;
			}else{
				data->state1 = STATE_A;
				buf1 = data->buffer_a;
			}
			data->phase = 0;
			data->GiveTask();
		}
	}
	return finished;
}


//------------------------------------------------------------
//    
//------------------------------------------------------------
int ToPortAudio::init_pa()
{
	if(initialized_portaudio) return -1;
	int er1;
	er1 = Pa_Initialize();
	phase = 0;
	outputParameters.device = Pa_GetDefaultOutputDevice();
	outputParameters.channelCount = 2;
	outputParameters.sampleFormat = paFloat32;
	outputParameters.suggestedLatency =
		Pa_GetDeviceInfo( outputParameters.device )->defaultLowOutputLatency;
	outputParameters.hostApiSpecificStreamInfo = NULL;
	int er2 = Pa_OpenStream(
		&stream,
		0,
		&outputParameters,
		GetSampleRate(),
		frames_per_buffer,
		paClipOff,
		ToPortAudio::PortAudioCallback,
		this);
	if((er1!=-1)&&(er2!=-1)){
		initialized_portaudio = true;
		return 1;
	}else{
		return -1;
	}
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
int ToPortAudio::start_pa()
{
	if(initialized_portaudio==false) return -1;
	int err = Pa_StartStream( stream );
	playing = true;
	return err;
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
int ToPortAudio::stop_pa()
{
	if((!initialized_portaudio)&&(!playing)) return -1;
	int err = Pa_StopStream( stream );
	return err;
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
int ToPortAudio::close_pa()
{
	if(initialized_portaudio==false) return -1;
	int err = Pa_CloseStream( stream );
	initialized_portaudio = false;
	playing = false;
	return err;
}

//------------------------------------------------------------
//    
//------------------------------------------------------------
int ToPortAudio::GetPhase() const
{
	return phase;
}
