//-----------------------------------------------------------
//    vco.cpp
//-----------------------------------------------------------
#include "vco.h"
#include "eg.h"
#include <math.h>
//-----------------------------------------------------------
//    
//-----------------------------------------------------------
const float default_frequency = 1000.0;
float* SynthVCO::sin_wave = 0;
float* SynthVCO::tri_wave = 0;
float* SynthVCO::saw_wave = 0;
const unsigned int SynthVCO::size_table = 10240;
//-----------------------------------------------------------
//    
//-----------------------------------------------------------
SynthVCO::SynthVCO(int id1)
	: SignalProcessBase(id1)
{
	phase_position = 0;
	SetFrequency(default_frequency);
	wtype = SIN;
	eg1 = new SynthEG();
	SetWaveTable();
	SetSignalProcessType(VCO);
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
SynthVCO::~SynthVCO()
{
	if(!ExistOtherVCO()){
		delete [] sin_wave;
		delete [] tri_wave;
		delete [] saw_wave;
		sin_wave = 0;
		tri_wave = 0;
		saw_wave = 0;
	}
	delete eg1;
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void SynthVCO::Work()
{
	float* table1;
	unsigned int bsz = GetSizeBuffer();
	float btmp[bsz];
	float c1;
	
	switch(wtype){
		case SIN:
			table1 = sin_wave;
			break;
		case TRI:
			table1 = tri_wave;
			break;
		case SAW:
			table1 = saw_wave;
			break;
		default:
			break;
	}
	
	int oncnt = GetNoteOnCount();
	int offcnt = GetNoteOffCount();
	
	if(oncnt >= 0){
		SetNoteOnCount(-1);
		for(int i1 = 0; i1 < bsz; i1++){
			if(i1 < oncnt){
				btmp[i1] = 0.0;
			}else if(i1 == oncnt){
				eg1->SetEGState(ATTACK);
				phase_position = 0;
				btmp[i1] = table1[0];
				c1 = delta * 0.5 + (delta * 0.5) * eg1->GetValue();
				phase_position += c1;
			}else if(i1 > oncnt){
				if(i1==offcnt){
					eg1->SetEGState(RELEASE);
					SetNoteOffCount(-1);
				}
				btmp[i1] = table1[(unsigned int)phase_position];
				c1 = delta * 0.5 + (delta * 0.5) * eg1->GetValue();
				phase_position += c1;
				if(phase_position >= (float)size_table){
					phase_position -= (float)size_table;
				}
			}
		}
		WriteBuffer(btmp);
	
	}else if((eg1->GetEGState()!=NON)){
		for(int i1 = 0; i1 < bsz; i1++){
			if(i1==offcnt){
				eg1->SetEGState(RELEASE);
				SetNoteOffCount(-1);
			}
			btmp[i1] = table1[(unsigned int)phase_position];
			c1 = delta * 0.5 + (delta * 0.5) * eg1->GetValue();
			phase_position += c1;
			if(phase_position >= (float)size_table){
				phase_position -= (float)size_table;
			}
		}
		WriteBuffer(btmp);
	
	}else{
		InitializeBuffer();
	}
	GiveTask();
	AccomplishTask();
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void SynthVCO::SetFrequency(float f1)
{
	if(f1<=0.0) return;
	frequency = f1;
	delta = (float)size_table * frequency / (float)GetSampleRate();
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void SynthVCO::SetWaveTable()
{
	if((sin_wave==0)||(tri_wave==0)||(saw_wave==0)){
		delete [] sin_wave;
		delete [] tri_wave;
		delete [] saw_wave;
		sin_wave = new float[size_table];
		tri_wave = new float[size_table];
		saw_wave = new float[size_table];
		
		float ctmp1 = 2.0 * M_PI / float(size_table);
		float ctmp2 = 2.0 / float(size_table);
		float ctmp3 = 4.0 / float(size_table);
		unsigned int mid = size_table / 2;
		for(int i1 = 0; i1 < size_table; i1++){;
			sin_wave[i1] = sin(ctmp1 * float(i1));
			saw_wave[i1] = - 1.0 + ctmp2 * float(i1);
			if(i1<mid) tri_wave[i1] = - 1.0 + ctmp3 * float(i1);
			else tri_wave[i1] = 1.0 - ctmp3 * float(i1 - mid);
		}
	}
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
bool SynthVCO::ExistOtherVCO() const
{
	int spb_sz = GetSignalProcesses().size();
	int idtmp, thisid = this->GetId();
	for(int i1 = 0; i1 < spb_sz; i1++){
		idtmp = GetSignalProcesses()[i1]->GetId();
		if(idtmp!=thisid){
			if(GetSignalProcesses()[i1]->GetType()==VCO) return true;
		}
	}
	return false;
}
